import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
//import java.util.*;
import java.io.*;

/**
 * @author Matt Spear
 *
 *The GUI front-end to the simulator of the UTM as specified.  An application
 * @see UTM
 */
public class FSASimApplet extends JFrame implements ActionListener
{
	private static final int MAXMOVES = 1000000;
	JButton go, step, dump, reset;
	JTextField input, output;
	JNumberTextField maxN;
	JFileChooser jfc;
	JMenuBar jmb;
	UTM fs;
	int max = -1, numAt = 0;
	// Temp?
	JTextArea ta;
	
	/**
	 * Simple constructor to set up the JFrame.
	 *
	 */
	public FSASimApplet()
	{
		Container cp = getContentPane();
		
		go = new JButton("Go");
		go.addActionListener(this);
		go.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		go.setEnabled(false);
		step = new JButton("Step");
		step.addActionListener(this);
		step.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		step.setEnabled(false);
		dump = new JButton("Dump");
		dump.addActionListener(this);
		dump.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		dump.setEnabled(false);
		reset = new JButton("Reset");
		reset.addActionListener(this);
		reset.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		reset.setEnabled(false);
		
		input = new JTextField(20);
		output = new JTextField(20);
		output.setEditable(false);
		maxN = new JNumberTextField(20);
		maxN.setText("" + MAXMOVES);
		
		ta = new JTextArea(10, 20);
		ta.setWrapStyleWord(true);
		ta.setEditable(false);
		JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JScrollPane pp = new JScrollPane(ta);
		
		jfc = new JFileChooser();
		
		jmb = new JMenuBar();
		JMenu jm = new JMenu("File");
		jm.setMnemonic(KeyEvent.VK_F);
		JMenuItem jmi = new JMenuItem("Open");
		jmi.addActionListener(this);
		jm.add(jmi);
		jm.addSeparator();
		jmi = new JMenuItem("Save As");
		jmi.setEnabled(false);
		jmi.addActionListener(this);
		jm.add(jmi);
		jm.addSeparator();
		jmi = new JMenuItem("Exit");
		jmi.addActionListener(this);
		jm.add(jmi);
		jmb.add(jm);
		
		fs = new UTM();
		
		JPanel l = new JPanel();
		JPanel r = new JPanel();
		r.setLayout(new BoxLayout(r, BoxLayout.Y_AXIS));
		JPanel gs = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel gin = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel gmm = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel grd = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel gou = new JPanel(new FlowLayout(FlowLayout.CENTER));
		
		r.add(new JLabel("Input:"));
		gin.add(input); 
		r.add(gin);
		r.add(new JLabel("Max Moves:")); 
		gmm.add(maxN); 
		r.add(gmm);
		r.add(new JLabel("Output:"));
		gou.add(output);
		r.add(gou);
		gs.add(go); gs.add(step); 
		r.add(gs);
		grd.add(reset); grd.add(dump);
		r.add(grd);
		
		cp.add(jmb, BorderLayout.NORTH);
		cp.add(pp, BorderLayout.CENTER); cp.add(r, BorderLayout.EAST);
	}
	
	/**
	 * Listen for the events as they come in.
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object o = e.getSource();
		if(o == go)
		{
			ta.setText("");
			int i = 0;
			if(maxN.getText().equals(""))
				max = MAXMOVES;
			else
			{	
				try
				{
					max = Integer.parseInt(maxN.getText());
				}
				catch(Exception ew)
				{
					ta.setText("Number too large " + ew.getMessage());
					return;
				}
			}
			if(input.getText().equals(""))
			{
				ta.setText("Must Have input");
				return;
			}
			try
			{
				fs.setInput(input.getText());
			}
			catch(UTMException dfse)
			{
				ta.setText(dfse.getMessage());
				return;
			}
			ta.append(fs.dump());
			while(!fs.isDone() && i < max)
			{	
				i++;
				try
				{
					fs.step();
					ta.append("\n" + fs.dump());
				}
				catch(UTMException ee) { }
				catch(RejectionException ee)
				{
					ta.append("\n" + ee.getMessage());
					go.setEnabled(false);
					step.setEnabled(false);
					dump.setEnabled(false);
					return;
				}
			}
			if(fs.isDone())
				ta.append("\n" + "Accepted Input");
			if(i == max)
				ta.append("\nToo many moves, try with a higher number");
			go.setEnabled(false);
			step.setEnabled(false);
			output.setText(fs.getOutput());
		}
		else if(o == step)
		{	
			if(numAt == max || fs.isDone())
			{
				go.setEnabled(false);
				step.setEnabled(false);
				if(fs.isDone())
					ta.append("\n" + "Accepted Input");
				else
					ta.append("\nToo many moves, try with a higher number");
				return;
			}
			numAt++;
			if(maxN.getText().equals(""))
				max = MAXMOVES;
			else
			{	
				try
				{
					max = Integer.parseInt(maxN.getText());
				}
				catch(Exception ew)
				{
					ta.setText("Number too large " + ew.getMessage());
					return;
				}
			}
			if(!fs.hasInput())
			{	
				if(input.getText().equals(""))
				{
					ta.setText("Must Have input");
					return;
				}
				try
				{
					fs.setInput(input.getText());
				}
				catch(UTMException dfse)
				{
					ta.setText(dfse.getMessage());
					return;
				}
			}
			if(ta.getText().equals(""))
			{	
				ta.append(fs.dump());
				return;
			}
			try
			{
				fs.step();
				ta.append("\n" + fs.dump());
			}
			catch(UTMException ee) { }
			catch(RejectionException ee)
			{
				ta.append("\n" + ee.getMessage());
				step.setEnabled(false);
				return;
			}
			output.setText(fs.getOutput());
			if(fs.isDone())
			{
				go.setEnabled(false);
				step.setEnabled(false);
				ta.append("\n" + "Accepted Input");
				return;
			}
		}
		else if(o == dump)
		{
			ta.setText(fs.toString());
		}
		else if(o == reset)
		{
			go.setEnabled(true);
			step.setEnabled(true);
			reset.setEnabled(true);
			dump.setEnabled(true);
			ta.setText("");
			fs.reset();
			numAt = 0;
		}
		else if(e.getActionCommand().equalsIgnoreCase("open"))
		{
			if(jfc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				File s = jfc.getSelectedFile();
				boolean b = true;
				try
				{
					parser p = new parser(new Scanner(new FileReader(s)));
					fs =(UTM) p.parse().value;
					b = false;
					go.setEnabled(true);
					dump.setEnabled(true);
					step.setEnabled(true);
					reset.setEnabled(true);
					ta.setText("");
					JMenu jm = jmb.getMenu(0);
					JMenuItem jmi = jm.getItem(2);
					if(jmi != null)
						jmi.setEnabled(true);
				}
				catch(NullPointerException ee)
				{
					ta.setText("Unable to parse input file.");
				}
				catch(Exception ea)
				{
					ta.setText(ea.toString());
				}
				catch(Error eee)
				{
					ta.setText(eee.toString());
				}
				finally
				{
					if(b)
					{	
						fs = null;
						go.setEnabled(false);
						dump.setEnabled(false);
						step.setEnabled(false);
						reset.setEnabled(false);
					}
				}
			}
		}
		else if(e.getActionCommand().equalsIgnoreCase("save as"))
		{
			if(jfc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				File f = jfc.getSelectedFile();
				try
				{
					BufferedWriter br = new BufferedWriter(new FileWriter(f));
					br.write(fs.toString());
					br.close();
				}
				catch(Exception ee)
				{
					ta.setText(ee.getMessage());
				}
			}
		}
		else if(e.getActionCommand().equalsIgnoreCase("exit"))
			dispose();
	}
	
	/**
	 * Run this.
	 * @param args Not used.
	 */
	public static void main(String args[])
	{
		FSASimApplet f = new FSASimApplet();
		f.setTitle("UTM Simulator");
		f.setSize(640,800);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JFrame.setDefaultLookAndFeelDecorated(true);
		f.pack();
		f.setVisible(true);
	}
}
