import javax.swing.*;
import javax.swing.text.*;

/**
 * @author Matt Spear
 *
 * Extremely simple class to disallow any non-numeric
 * characters being entered in a JTextField.
 */
public class JNumberTextField extends JTextField
{
	public JNumberTextField() { super(); }
	public JNumberTextField(int i) { super(i); }
	/**
	 * Gets the DocumentModel
	 */
	protected Document createDefaultModel() 
	{
		return new NumberCaseDocument();
	}
	/**
	 * Custom Document model disallowing the non-numeric characters.
	 * @author Matt Spear
	 */
	class NumberCaseDocument extends PlainDocument 
	{
		StringBuffer sb;
		public void insertString(int offs, String str, AttributeSet a) throws BadLocationException 
		{
			if (str == null) 
				return;
			sb = new StringBuffer(str);
			for (int i = 0; i < str.length(); i++) 
			{
				if(!(sb.charAt(i) >= '0' && sb.charAt(i) <= '9'))
					sb.deleteCharAt(i);
			}
			super.insertString(offs, sb.toString(), a);
		}
	}
}
