/**
 * @author Matt Spear
 *
 * Simple class that represents a Node.
 * 
 * @see Edge
 * @see Graph
 */
public class Node implements Comparable 
{
	private String name;
	private boolean visited;
	private boolean active; // Not used, kept for Swing GUI that was never implemented
	private boolean accepting;
	private boolean root;
	
	/**
	 * Constructs a default Node, no name not visited, not active, not accepting and not root
	 *
	 */
	public Node()
	{
		name = "";
		visited = false;
		active = false;
		accepting = false;
		root = false;
	}
	/**
	 * Constructs a default Ndoe with the name specified
	 * @param name the name of the Node
	 */
	public Node(String name)
	{
		this.name = name;
		visited = false;
		active = false;
		accepting = false;
		root = false;
	}
	/**
	 * Constructs a default Node with name, accepting and root specified.
	 * @param name the name of the Node
	 * @param accepting if the node is an accepting node
	 * @param root if the node is a root
	 */
	public Node(String name, boolean accepting, boolean root)
	{
		this.name = name;
		this.accepting = accepting;
		this.root = root;
		visited = false;
		active = false;
	}
	/**
	 * Constructs a Node with everything specified
	 * @param name th name of the node
	 * @param visited if the node has been visited
	 * @param active if the node is the Node currently active
	 * @param accepting if the node is accepting
	 * @param root if the node is root
	 */
	public Node(String name, boolean visited, boolean active, boolean accepting, boolean root)
	{
		this.name = name;
		this.visited = visited;
		this.accepting = accepting;
		this.active = active;
		this.root = root;
	}
	
	/**
	 * Gets the name of the Node
	 * @return the name of the Node
	 */
	public String getName() { return name; }
	/**
	 * Tests if the Node has been visited
	 * @return true iff the node has been visited
	 */
	public boolean isVisited() { return visited; }
	/**
	 * Tests if teh node is the active node
	 * @return true iff the node is active
	 */
	public boolean isActive() { return active; }
	/**
	 * Tests if the node is an accepting node
	 * @return true iff the node is an accepting node
	 */
	public boolean isAccepting() { return accepting; }
	/**
	 * Tests if the node is a root 
	 * @return true iff the node is a root
	 */
	public boolean isRoot() { return root; }
	/**
	 * Sets the name of the node
	 * @param name the name of the node
	 */
	public void setName(String name) { this.name = name; }
	/**
	 * Visites the node, makes isVisited return false
	 *
	 */
	public void visit() { visited = true; }
	/**
	 * Unvisits a node, makes isVisited return false
	 *
	 */
	public void unVisit() { visited = false; }
	/**
	 * Activates a node, makes isActive return true
	 *
	 */
	public void activate() { active = true; }
	/**
	 * Deactivates a node, makes isActive return fals
	 *
	 */
	public void deactivate() { active = false; }
	/**
	 * Makes the node an accepting node.
	 *
	 */
	public void makeAccepting() { accepting = true; }
	/**
	 * Makes the noe a non-accepting node
	 *
	 */
	public void makeNonAccepting() { accepting = false; }
	/**
	 * Makes the node a root.
	 *
	 */
	public void makeRoot() { root = true; }
	/**
	 * Makes the node a non-root.
	 *
	 */
	public void makeNonRoot() { root = false; }
	
	/**
	 * Tests for equality.
	 */
	public boolean equals(Object o)
	{
		if(o instanceof Node)
			return equals((Node)o);
		return false;
	}
	/**
	 * Checks to see if the name of the Nodes are equivilant
	 * @param n the node to compare against
	 * @return true iff the names are equivilant
	 */
	public boolean equals(Node n)
	{
		return n.name.equals(name);
	}
	/**
	 * Checks to see if the name of the Nodes are equivilant
	 * @param n the node to compare against
	 * @return true iff the names are equivilant
	 */
	public boolean equals(String s)
	{
		return name.equals(s);
	}
	/**
	 * Compares to another Node, if the Node specified
	 * is root returns 1, else returns the comparison of the names.
	 */
	public int compareTo(Object o)
	{
		if(((Node)o).isRoot())
			return 1;
		return name.compareTo(((Node)o).name);
	}
	
	/**
	 * Returns the name of this node.
	 */
	public String toString()
	{
		return name;
	}
	/**
	 * Returns a clone of this.
	 */
	public Object clone()
	{
		return new Node(name, visited, active, accepting, root);
	}
}
